{% macro messages() %}
    {% with messages = get_flashed_messages(with_categories=true) %}
        {% for category, message in messages %}
        <div class="ui attached message {{ category }}">
            {{ message }}
        </div>
        {% endfor %}
    {% endwith %}
{% endmacro %}
