{% extends theme('base.html') %}
{% import 'fomanticui.html' as sui %}

{% block content %}
    <div class="ui clearing segment">
        <a href="{{ url_for('account.index') }}">
            <img class="ui tiny centered image" src="/static/img/canaille-head.png" alt="{{ website_name }}">
        </a>

        <h2 class="ui center aligned header">
            <div class="content">
                {{ _("About canaille") }}
            </div>
            <div class="sub header">{% trans %}Free and open-source identity provider.{% endtrans %}</div>
        </h2>
        <p style="text-align: center">
            <a href="https://pypi.org/project/canaille/">{{ gettext("Version %(version)s", version=version) }}</a> ·
            <a href="https://canaille.yaal.coop">{% trans %}Homepage{% endtrans %}</a> ·
            <a href="https://canaille.readthedocs.io">{% trans %}Documentation{% endtrans %}</a> ·
            <a href="https://gitlab.com/yaal/canaille">{% trans %}Source code{% endtrans %}</a>
        </p>

    </div>
{% endblock %}
