{% extends theme('base.html') %}
{% import 'fomanticui.html' as sui %}

{% block content %}
    <div class="loginform">
        <h3 class="ui top attached header">
            {% if logo_url %}
                <img class="ui image" src="{{ logo_url }}">
            {% endif %}
            <div class="content">
                {% trans %}Password reset{% endtrans %}
            </div>
        </h3>

        <div class="ui attached clearing segment">
            {{ sui.render_form(form, _("Password reset"), action=url_for("account.reset", uid=uid, hash=hash)) }}
        </div>
    </div>
{% endblock %}
