{% extends theme('base.html') %}
{% import 'fomanticui.html' as sui %}

{% block content %}
    <div class="ui clearing segment">
        {% if logo_url %}
            <a href="{{ url_for('account.index') }}">
                <img class="ui tiny centered image" src="{{ logo_url }}" alt="{{ website_name }}">
            </a>
        {% else %}
            <i class="massive sign in icon image ui"></i>
        {% endif %}

        <h2 class="ui center aligned header">
            <div class="content">
                {{ _("Sign in as %(username)s", username=username) }}
            </div>
            <div class="sub header">{% trans %}Please enter your password for this account.{% endtrans %}</div>
        </h2>

        <form method="POST"
            id="{{ form.id or form.__class__.__name__|lower }}"
            action="{{ request.url }}"
            role="form"
            class="ui form"
        >

            {{ form.hidden_tag() if form.hidden_tag }}
            {{ sui.render_field(form.password, icon="key", noindicator=true) }}

            <div class="ui right aligned container">
                <div class="ui stackable buttons">
                    <a type="button" class="ui right floated button" href="{{ url_for('account.login') }}">{{ _("I am not %(username)s", username=username) }}</a>
                    {% if has_smtp and has_password_recovery %}
                        <a type="button" class="ui right floated button" href="{{ url_for('account.forgotten') }}">{{ _("Forgotten password") }}</a>
                    {% endif %}
                    <button type="submit" class="ui right floated primary button">{{ _("Sign in") }}</button>
                </div>
            </div>
        </form>
    </div>
{% endblock %}
